﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Infragistics.Olap;
using Infragistics.Olap.Data.Base;
using System.ComponentModel;
using Infragistics.Olap.Data;

namespace Pivot_Analysis
{
    public static class Extensions
    {
        /// <summary>
        /// Asynchronously loads and expands <see cref="IFilterViewModel.FilterMembers"/> hierarchy up to the specified level.
        /// </summary>
        /// <param name="olapViewModel">The olap view model.</param>
        /// <param name="filterViewModel">The <see cref="IFilterViewModel"/> whose <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded.</param>
        /// <param name="level">The level which specifies the depth of <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded up to.</param>
        /// <returns>The <see cref="Task"/> which completes when all members of <see cref="IFilterViewModel.FilterMembers"/> hierarchy are loaded and set to expanded up to the specified level depth.</returns
        public static Task ExpandToLevelAsync(this IOlapViewModel olapViewModel, IFilterViewModel filterViewModel, ILevel level)
        {
            if (level == null || filterViewModel.ViewModel != olapViewModel)
            {
                return null;
            }

            Action action = () =>
            {
                ExpandToLevel(filterViewModel.FilterMembers, level.Depth);
                FilterViewModelBase filterViewModelBase = filterViewModel as FilterViewModelBase;
                if (filterViewModelBase != null)
                {
                    filterViewModelBase.IsDirty = true;
                }
            };

            Task expandTask = new Task(action);

            if (((Level)level).IsLoaded && level.Members.Count > 0)
            {
                expandTask.RunSynchronously();
            }
            else
            {
                EventHandler<AsyncCompletedEventArgs> callback = null;
                callback = (s, args) =>
                {
                    IHierarchy hierarchy = args.UserState as IHierarchy;
                    if (hierarchy == null)
                    {
                        return;
                    }

                    filterViewModel.LoadFilterMembersCompleted -= callback;
                    if (expandTask.Status == TaskStatus.Created)
                    {
                        expandTask.RunSynchronously();
                    }
                };

                filterViewModel.AttachToProvider();
                filterViewModel.LoadFilterMembersCompleted += callback;
                filterViewModel.DataProvider.LoadMembersAsync(olapViewModel.Cube, level.ParentHierarchy);
                return expandTask;
            }

            return expandTask;
        }

        /// <summary>
        /// Asynchronously loads and expands <see cref="IFilterViewModel.FilterMembers"/> hierarchy up to depth of the level specified by its name.
        /// </summary>
        /// <param name="olapViewModel">The olap view model.</param>
        /// <param name="filterViewModel">The <see cref="IFilterViewModel"/> whose <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded.</param>
        /// <param name="levelUniqueName">The unique name of the level which specifies the depth of <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded up to.</param>
        /// <returns>The <see cref="Task"/> which completes when all members of <see cref="IFilterViewModel.FilterMembers"/> hierarchy are loaded and set to expanded up to the specified level depth.</returns>
        public static Task ExpandToLevelAsync(this IOlapViewModel olapViewModel, IFilterViewModel filterViewModel, string levelUniqueName)
        {
            ILevel level = filterViewModel.Hierarchy.Levels.FirstOrDefault(l => l.UniqueName == levelUniqueName);
            return olapViewModel.ExpandToLevelAsync(filterViewModel, level);
        }

        /// <summary>
        /// Asynchronously loads and expands <see cref="IFilterViewModel.FilterMembers"/> hierarchy up to the specified level depth.
        /// </summary>
        /// <param name="olapViewModel">The olap view model.</param>
        /// <param name="filterViewModel">The <see cref="IFilterViewModel"/> whose <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded.</param>
        /// <param name="levelDepth">The level depth of <see cref="IFilterViewModel.FilterMembers"/> hierarchy will be expanded up to.</param>
        /// <returns>The <see cref="Task"/> which completes when all members of <see cref="IFilterViewModel.FilterMembers"/> hierarchy are loaded and set to expanded up to the specified level depeth.</returns>
        public static Task ExpandToLevelAsync(this IOlapViewModel olapViewModel, IFilterViewModel filterViewModel, int levelDepth)
        {
            if (levelDepth < filterViewModel.Hierarchy.Levels.Count)
            {
                ILevel level = filterViewModel.Hierarchy.Levels[levelDepth];
                return olapViewModel.ExpandToLevelAsync(filterViewModel, level);
            }

            return null;
        }

        private static void ExpandToLevel(IEnumerable<IFilterMember> filterMembers, int levelDepth)
        {
            foreach (IFilterMember filterMember in filterMembers)
            {
                if (filterMember.Member.LevelDepth < levelDepth)
                {
                    filterMember.IsExpanded = true;
                    if (filterMember.Member.LevelDepth < levelDepth - 1)
                    {
                        ExpandToLevel(filterMember.FilterMembers, levelDepth);
                    }
                }
            }
        }
    }
}
